import os
from subprocess import call
import unittest

from testpath.commands import *

class CommandsTests(unittest.TestCase):
    def test_assert_calls(self):
        initial_path = os.environ['PATH']        
        
        with assert_calls('foobar'):
            call(['foobar'])
        
        with self.assertRaises(AssertionError):
            with assert_calls('foo'):
                pass
        
        # The context manager should clean up $PATH again
        self.assertEqual(os.environ['PATH'], initial_path)
    
    def test_assert_calls_with_args(self):
        with assert_calls('foo', ['bar', 'baz']):
            call(['foo', 'bar', 'baz'])
        
        with self.assertRaises(AssertionError):
            with assert_calls('cheese', ['crackers']):
                call(['cheese', 'biscuits'])
                call(['cheese', 'wine'])

    def test_assert_calls_twice(self):
        with assert_calls('git'):
            call(['git'])

        with self.assertRaises(AssertionError):
            with assert_calls('git'):
                pass



import os
import unittest

try:
    import pathlib
except ImportError:
    # Python 2 backport
    import pathlib2 as pathlib

from testpath.asserts import *
from testpath.tempdir import TemporaryDirectory

class TestAssertFunctions(unittest.TestCase):
    def setUp(self):
        self.td = TemporaryDirectory()
        self.addCleanup(self.td.cleanup)
        
        self.file_path = os.path.join(self.td.name, 'afile')
        with open(self.file_path, 'w') as f:
            f.write('Blah')
        
        self.dir_path = os.path.join(self.td.name, 'adir')
        os.mkdir(self.dir_path)
        
        self.link_path = os.path.join(self.td.name, 'alink')
        if os.name == 'posix':
            # Symlinks are rarely usable on Windows, because a special
            # permission is needed to create them.
            os.symlink(self.file_path, self.link_path)
        
        self.nonexistant_path = os.path.join(self.td.name, 'doesntexist')
    
    def test_exists(self):
        assert_path_exists(self.file_path)
        assert_path_exists(pathlib.Path(self.file_path))
        assert_not_path_exists(self.nonexistant_path)
        
        with self.assertRaises(AssertionError):
            assert_path_exists(self.nonexistant_path)
        
        with self.assertRaises(AssertionError):
            assert_not_path_exists(self.file_path)
    
    def test_isfile(self):
        assert_isfile(self.file_path)
        assert_not_isfile(self.dir_path)
        
        with self.assertRaises(AssertionError):
            assert_isfile(self.dir_path)
    
        with self.assertRaises(AssertionError):
            assert_not_isfile(self.file_path)

    def test_isfile_symlink(self):
        if os.name == 'nt':
            raise unittest.SkipTest('symlink')
        assert_isfile(self.link_path)  # Follows the link by default
        assert_not_isfile(self.link_path, follow_symlinks=False)

        with self.assertRaises(AssertionError):
            assert_isfile(self.link_path, follow_symlinks=False)
        
        with self.assertRaises(AssertionError):
            assert_not_isfile(self.link_path)

    def test_isdir(self):
        assert_isdir(self.dir_path)
        assert_isdir(pathlib.Path(self.dir_path))
        assert_not_isdir(self.file_path)
        
        with self.assertRaises(AssertionError):
            assert_isdir(self.file_path)
        
        with self.assertRaises(AssertionError):
            assert_not_isdir(self.dir_path)

    def test_islink(self):
        if os.name == 'nt':
            raise unittest.SkipTest('symlink')
        assert_islink(self.link_path, to=self.file_path)
        assert_islink(pathlib.Path(self.link_path),
                      to=pathlib.Path(self.file_path))
        assert_not_islink(self.file_path)
        
        with self.assertRaises(AssertionError) as c:
            assert_islink(self.file_path)
        self.assertIn('not a symlink', str(c.exception))
        
        with self.assertRaises(AssertionError) as c:
            assert_islink(self.link_path, to=self.dir_path)
        self.assertIn('target of', str(c.exception))
        
        with self.assertRaises(AssertionError):
            assert_not_islink(self.link_path)



import os
import unittest

import testpath


class EnvironmentUtilsTests(unittest.TestCase):
    def setUp(self):
        # We're actually using this for its specified purpose, rather than
        # explicitly testing it.
        self.addCleanup(testpath.make_env_restorer())

    def test_temporary_env(self):
        os.environ['abc123'] = '4'
        os.environ['def567'] = '8'

        with testpath.temporary_env({'abc123': '9'}):
            self.assertEqual(os.environ['abc123'], '9')
            self.assertNotIn('def567', os.environ)
            os.environ['foo951'] = 'bar'

        self.assertEqual(os.environ['abc123'], '4')
        self.assertEqual(os.environ['def567'], '8')
        self.assertNotIn('foo951', os.environ)

    def test_modified_env(self):
        os.environ['abc123'] = '4'
        os.environ['def567'] = '8'
        os.environ['ghi789'] = '10'
        os.environ.pop('foo951', None)

        with testpath.modified_env({'abc123': '9', 'def567': None}):
            self.assertEqual(os.environ['abc123'], '9')
            self.assertNotIn('def567', os.environ)
            self.assertEqual(os.environ['ghi789'], '10') # Not affected
            os.environ['foo951'] = 'bar'

        self.assertEqual(os.environ['abc123'], '4')
        self.assertEqual(os.environ['def567'], '8')
        self.assertEqual(os.environ['ghi789'], '10') # Not affected
        self.assertNotIn('foo951', os.environ)

    def test_modified_env_nosnapshot(self):
        os.environ['abc123'] = '4'
        os.environ['def567'] = '8'
        os.environ['ghi789'] = '10'
        os.environ.pop('foo951', None)

        with testpath.modified_env({'abc123': '9', 'def567': None}, snapshot=False):
            self.assertEqual(os.environ['abc123'], '9')
            self.assertNotIn('def567', os.environ)
            self.assertEqual(os.environ['ghi789'], '10') # Not affected
            os.environ['foo951'] = 'bar'

        self.assertEqual(os.environ['abc123'], '4')
        self.assertEqual(os.environ['def567'], '8')
        self.assertEqual(os.environ['ghi789'], '10')
        self.assertEqual(os.environ['foo951'], 'bar') # Not reset
